VALIDATOR_HEADER = "X-Chutes-Validator"
MINER_HEADER = "X-Chutes-Miner"
NONCE_HEADER = "X-Chutes-Nonce"
SIGNATURE_HEADER = "X-Chutes-Signature"
HOTKEY_HEADER = "X-Chutes-Hotkey"

# Environment variable names for CLI options (when not passed as flags)
HOTKEY_ENVVAR = "HOTKEY"
MINER_API_ENVVAR = "MINER_API"
VALIDATOR_API_ENVVAR = "VALIDATOR_API"
LAUNCH_JWT_ENVVAR = "CHUTES_LAUNCH_JWT"
